<?php

namespace Drupal\acquia_search\v3;

use Drupal\acquia_search\AcquiaSearchSolrApiBase;
use Drupal\acquia_search\AcquiaSearchSolrApiInterface;
use Drupal\acquia_search\AcquiaSearchSolrMessages;

/**
 * Class AcquiaSearchApi.
 */
class AcquiaSearchSolrApi extends AcquiaSearchSolrApiBase implements AcquiaSearchSolrApiInterface {

  /**
   * Define the Acquia Search Version.
   */
  const ACQUIA_SEARCH_VERSION = 3;

  /**
   * {@inheritdoc}
   */
  protected $version = self::ACQUIA_SEARCH_VERSION;

  /**
   * Wrapper for constructor.
   *
   * @param string $subscription
   *   Subscription name.
   * @param string $api_key
   *   API key.
   * @param string $host
   *   Search host name.
   * @param string $uuid
   *   Application UUID.
   *
   * @return \Drupal\acquia_search\AcquiaSearchSolrApiInterface
   *   API instance.
   *
   * @throws \InvalidArgumentException
   */
  public static function get($subscription, $api_key, $host, $uuid) {
    return new self($subscription, $api_key, $host, $uuid);
  }

  /**
   * Get API from settings.
   *
   * @return \Drupal\acquia_search\AcquiaSearchSolrApiInterface
   *   API instance.
   *
   * @throws \InvalidArgumentException
   */
  public static function getFromSettings() {
    $subscription = acquia_agent_settings('acquia_identifier');
    $api_key = acquia_agent_settings('acquia_key');
    $host = variable_get('acquia_search_v' . self::ACQUIA_SEARCH_VERSION . '_host', 'https://api.sr-prod02.acquia.com');
    // If no UUID explicitly set, get it from subscription data.
    $uuid = acquia_agent_settings('acquia_application_uuid');

    return new self($subscription, $api_key, $host, $uuid);
  }

  /**
   * Returns list of search indexes.
   *
   * @return array
   *   Search indexes list.
   */
  public function getCores() {
    $result = &drupal_static(__FUNCTION__);

    if (!$result) {
      // Get data from cache.
      $cid = 'acquia_search.cores.' . $this->subscription;

      if (($cache = cache_get($cid, 'cache')) && $cache->expire > REQUEST_TIME) {
        return $cache->data;
      }
      $query = ['network_id' => $this->subscription];
      $nonce = \AcquiaSearchAuth::randomBytes(24);
      $config_path = '/v2/index/configure';
      $auth_string = sprintf('id=%s&nonce=%s&realm=search&version=2.0', $this->appUuid, $nonce);
      $host = preg_replace('(^https?://)', '', $this->host);
      $req_params = [
        'GET',
        $host,
        $config_path,
        drupal_http_build_query($query),
        $auth_string,
        REQUEST_TIME,
      ];

      $headers = [
        'Authorization' => $this->calculateAuthHeader($this->apiKey, $req_params, $auth_string),
        'X-Authorization-Timestamp' => REQUEST_TIME,
      ];

      // Set URL. Ignored because it went one col over 80.
      //@phpcs:ignore
      $url = url($this->host . $config_path, [
        'https' => TRUE,
        'query' => $query,
      ]);

      $options = ['headers' => $headers, 'timeout' => self::REQUEST_TIMEOUT];
      $response = drupal_http_request($url, $options);

      if ($response->code > 300 || !isset($response->data)) {
        $error_message = t("Couldn't connect to AcquiaSearchApi to get list of cores. Reason: @reason. Status code: @code. Request: @request", [
          '@reason' => $response->status_message ?? '',
          '@code' => $response->code ?? '',
          '@request' => $response->request ?? '',
        ]);
        watchdog('acquia_search_solr', $error_message, [], WATCHDOG_ERROR);
        // When API is not reachable, cache results for 1 minute.
        $expire = REQUEST_TIME + 60;
        cache_set($cid, [], 'cache', $expire);

        return [];
      }

      $result = $this->processResponse($response->data);

      // Cache will be set in both cases, 1. when search v3 cores are found and
      // 2. when there are no search v3 cores but api is reachable.
      $expire = REQUEST_TIME + (60 * 60 * 24);
      cache_set($cid, $result, 'cache', $expire);
    }
    return $result;
  }

  /**
   * Adjusts response structure.
   *
   * @param string $data
   *   Data in JSON format.
   *
   * @return array
   *   Search indexes list keyed by ID.
   */
  protected function processResponse($data) {
    $result = [];
    $indexes = json_decode($data, TRUE);

    if (is_array($indexes)) {
      foreach ($indexes as $index) {
        $result[$index['key']] = [
          'host' => $index['host'],
          'core_id' => $index['key'],
          'data' => $index,
        ];
      }
    }

    return $result;
  }

  /**
   * Calculates authorization headers.
   *
   * @param string $key
   *   API key.
   * @param array $params
   *   Request parameters.
   * @param string $auth_string
   *   Authorization string.
   *
   * @return string
   *   Authorization header.
   */
  private function calculateAuthHeader($key, array $params, $auth_string) {
    $auth_string = str_replace(['&', '='], ['",', '="'], $auth_string);
    $key = base64_decode($key, TRUE);
    $signature_base_string = implode(PHP_EOL, $params);

    $digest = hash_hmac('sha256', $signature_base_string, $key, TRUE);
    $signature = base64_encode($digest);
    $header = sprintf('acquia-http-hmac %s ",signature="%s"', $auth_string, $signature);

    return $header;
  }

  /**
   * {@inheritdoc}
   */
  public function getUrl($server_id) {
    $preferred_core_service = $this->getPreferredCoreService();
    if ($preferred_index = $preferred_core_service->getPreferredCore($server_id)) {
      return sprintf('https://%s', $preferred_index['data']['index_url']);
    }
    else {
      $message = AcquiaSearchSolrMessages::getNoPreferredIndexError(
        $preferred_core_service->getAvailableCoreIds()
      );
      drupal_set_message($message, 'warning');
      return NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getServiceClass($ecosystem) {
    return 'AcquiaSearchV3' . $ecosystem;
  }

  /**
   * {@inheritdoc}
   */
  public function getPossibleCores(&$possible_cores) {}

}
